@@ -33,10 +33,16 @@
 #  define VTKARROWGLYPHFILTER_DEPRECATED_NO_EXPORT VTKARROWGLYPHFILTER_NO_EXPORT VTKARROWGLYPHFILTER_DEPRECATED
 #endif
 
+/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
 #if 0 /* DEFINE_NO_DEPRECATED */
 #  ifndef VTKARROWGLYPHFILTER_NO_DEPRECATED
 #    define VTKARROWGLYPHFILTER_NO_DEPRECATED
 #  endif
 #endif
 
+/* VTK-HeaderTest-Exclude: vtkArrowGlyphFilterModule.h */
+
+/* Include ABI Namespace */
+#include "vtkABINamespace.h"
+
 #endif /* VTKARROWGLYPHFILTER_EXPORT_H */
