@@ -3,23 +3,27 @@ from vtkmodules.vtkCommonCore import vtkFloatArray, vtkUnsignedCharArray
 from vtkmodules.vtkCommonDataModel import vtkImageData
 from vtkmodules.vtkIOLegacy import vtkDataSetWriter
 
-from vtkmodules.web.camera import *
-from vtkmodules.web import iteritems, buffer
+from vtkmodules.web.camera import normalize, vectProduct, dotProduct
+from vtkmodules.web import iteritems
 
-import json, os, math, gzip, shutil, array
+import json, os, math, array
 
 # -----------------------------------------------------------------------------
 # Helper function
 # -----------------------------------------------------------------------------
 
+
 def getScalarFromRGB(rgb, scalarRange=[-1.0, 1.0]):
-    delta = (scalarRange[1] - scalarRange[0]) / 16777215.0 # 2^24 - 1 => 16,777,215
+    delta = (scalarRange[1] - scalarRange[0]) / 16777215.0  # 2^24 - 1 => 16,777,215
     if rgb[0] != 0 or rgb[1] != 0 or rgb[2] != 0:
         # Decode encoded value
-        return scalarRange[0] + delta * float(rgb[0]*65536 + rgb[1]*256 + rgb[2] - 1)
+        return scalarRange[0] + delta * float(
+            rgb[0] * 65536 + rgb[1] * 256 + rgb[2] - 1
+        )
     else:
         # No value
-        return float('NaN')
+        return float("NaN")
+
 
 def convertImageToFloat(srcPngImage, destFile, scalarRange=[0.0, 1.0]):
     reader = vtkPNGReader()
@@ -29,23 +33,22 @@ def convertImageToFloat(srcPngImage, destFile, scalarRange=[0.0, 1.0]):
     stackSize = rgbArray.GetNumberOfTuples()
     size = reader.GetOutput().GetDimensions()
 
-
     outputArray = vtkFloatArray()
     outputArray.SetNumberOfComponents(1)
     outputArray.SetNumberOfTuples(stackSize)
 
     for idx in range(stackSize):
         outputArray.SetTuple1(
-            idx,
-            getScalarFromRGB(rgbArray.GetTuple(idx), scalarRange)
+            idx, getScalarFromRGB(rgbArray.GetTuple(idx), scalarRange)
         )
 
     # Write float file
-    with open(destFile, 'wb') as f:
-        f.write(buffer(outputArray))
+    with open(destFile, "wb") as f:
+        f.write(memoryview(outputArray))
 
     return size
 
+
 def convertRGBArrayToFloatArray(rgbArray, scalarRange=[0.0, 1.0]):
     linearSize = rgbArray.GetNumberOfTuples()
 
@@ -55,26 +58,26 @@ def convertRGBArrayToFloatArray(rgbArray, scalarRange=[0.0, 1.0]):
 
     for idx in range(linearSize):
         outputArray.SetTuple1(
-            idx,
-            getScalarFromRGB(rgbArray.GetTuple(idx), scalarRange)
+            idx, getScalarFromRGB(rgbArray.GetTuple(idx), scalarRange)
         )
 
     return outputArray
 
+
 # -----------------------------------------------------------------------------
 # Composite.json To order.array
 # -----------------------------------------------------------------------------
 class CompositeJSON(object):
     def __init__(self, numberOfLayers):
         self.nbLayers = numberOfLayers
-        self.encoding = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+        self.encoding = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 
     def load(self, file):
         with open(file, "r") as f:
             composite = json.load(f)
             self.width = composite["dimensions"][0]
             self.height = composite["dimensions"][1]
-            self.pixels = composite["pixel-order"].split('+')
+            self.pixels = composite["pixel-order"].split("+")
             self.imageSize = self.width * self.height
             self.stackSize = self.imageSize * self.nbLayers
 
@@ -84,7 +87,7 @@ class CompositeJSON(object):
     def getStackSize(self):
         return self.stackSize
 
-    def writeOrderSprite(self,path):
+    def writeOrderSprite(self, path):
         ds = vtkImageData()
         ds.SetDimensions(self.width, self.height, self.nbLayers)
         ds.GetPointData().AddArray(self.getSortedOrderArray())
@@ -94,10 +97,9 @@ class CompositeJSON(object):
         writer.SetFileName(path)
         writer.Update()
 
-
     def getSortedOrderArray(self):
         sortedOrder = vtkUnsignedCharArray()
-        sortedOrder.SetName('layerIdx');
+        sortedOrder.SetName("layerIdx")
         sortedOrder.SetNumberOfTuples(self.stackSize)
 
         # Reset content
@@ -106,16 +108,18 @@ class CompositeJSON(object):
 
         idx = 0
         for pixel in self.pixels:
-            x = (idx % self.width)
-            y = (idx / self.width)
+            x = idx % self.width
+            y = idx / self.width
             flipYIdx = self.width * (self.height - y - 1) + x
-            if '@' in pixel:
+            if "@" in pixel:
                 idx += int(pixel[1:])
             else:
                 # Need to decode the order
                 layerIdx = 0
                 for layer in pixel:
-                    sortedOrder.SetValue(flipYIdx + self.imageSize * layerIdx, self.encoding.index(layer))
+                    sortedOrder.SetValue(
+                        flipYIdx + self.imageSize * layerIdx, self.encoding.index(layer)
+                    )
                     layerIdx += 1
 
                 # Move to next pixel
@@ -123,10 +127,12 @@ class CompositeJSON(object):
 
         return sortedOrder
 
+
 # -----------------------------------------------------------------------------
 # Composite Sprite to Sorted Composite Dataset Builder
 # -----------------------------------------------------------------------------
 
+
 class ConvertCompositeSpriteToSortedStack(object):
     def __init__(self, directory):
         self.basePath = directory
@@ -137,7 +143,7 @@ class ConvertCompositeSpriteToSortedStack(object):
         # Load JSON metadata
         with open(os.path.join(directory, "config.json"), "r") as f:
             self.config = json.load(f)
-            self.nbLayers = len(self.config['scene'])
+            self.nbLayers = len(self.config["scene"])
             while len(self.layers) < self.nbLayers:
                 self.layers.append({})
 
@@ -147,7 +153,7 @@ class ConvertCompositeSpriteToSortedStack(object):
         with open(os.path.join(directory, "offset.json"), "r") as f:
             offsets = json.load(f)
             for key, value in iteritems(offsets):
-                meta = key.split('|')
+                meta = key.split("|")
                 if len(meta) == 2:
                     self.layers[int(meta[0])][meta[1]] = value
                 elif meta[1] in self.layers[int(meta[0])]:
@@ -162,39 +168,43 @@ class ConvertCompositeSpriteToSortedStack(object):
 
     def convert(self):
         for root, dirs, files in os.walk(self.basePath):
-            if 'rgb.png' in files:
-                print ('Process', root)
+            if "rgb.png" in files:
+                print("Process", root)
                 self.processDirectory(root)
 
     def processDirectory(self, directory):
-        self.imageReader.SetFileName(os.path.join(directory, 'rgb.png'))
+        self.imageReader.SetFileName(os.path.join(directory, "rgb.png"))
         self.imageReader.Update()
         rgbArray = self.imageReader.GetOutput().GetPointData().GetArray(0)
 
-        self.composite.load(os.path.join(directory, 'composite.json'))
+        self.composite.load(os.path.join(directory, "composite.json"))
         orderArray = self.composite.getSortedOrderArray()
 
         imageSize = self.composite.getImageSize()
         stackSize = self.composite.getStackSize()
 
         # Write order (sorted order way)
-        with open(os.path.join(directory, 'order.uint8'), 'wb') as f:
-            f.write(buffer(orderArray))
-            self.data.append({'name': 'order', 'type': 'array', 'fileName': '/order.uint8'})
+        with open(os.path.join(directory, "order.uint8"), "wb") as f:
+            f.write(memoryview(orderArray))
+            self.data.append(
+                {"name": "order", "type": "array", "fileName": "/order.uint8"}
+            )
 
         # Encode Normals (sorted order way)
-        if 'normal' in self.layers[0]:
+        if "normal" in self.layers[0]:
             sortedNormal = vtkUnsignedCharArray()
-            sortedNormal.SetNumberOfComponents(3) # x,y,z
+            sortedNormal.SetNumberOfComponents(3)  # x,y,z
             sortedNormal.SetNumberOfTuples(stackSize)
 
             # Get Camera orientation and rotation information
-            camDir = [0,0,0]
-            worldUp = [0,0,0]
+            camDir = [0, 0, 0]
+            worldUp = [0, 0, 0]
             with open(os.path.join(directory, "camera.json"), "r") as f:
                 camera = json.load(f)
-                camDir = normalize([ camera['position'][i] - camera['focalPoint'][i] for i in range(3) ])
-                worldUp = normalize(camera['viewUp'])
+                camDir = normalize(
+                    [camera["position"][i] - camera["focalPoint"][i] for i in range(3)]
+                )
+                worldUp = normalize(camera["viewUp"])
 
             # [ camRight, camUp, camDir ] will be our new orthonormal basis for normals
             camRight = vectProduct(camDir, worldUp)
@@ -210,26 +220,45 @@ class ConvertCompositeSpriteToSortedStack(object):
             zPosCount = 0
             zNegCount = 0
             for layer in self.layers:
-                normalOffset = layer['normal']
+                normalOffset = layer["normal"]
                 for idx in range(imageSize):
                     normalByLayer.SetTuple3(
                         layerIdx * imageSize + idx,
-                        getScalarFromRGB(rgbArray.GetTuple(idx + normalOffset[0] * imageSize)),
-                        getScalarFromRGB(rgbArray.GetTuple(idx + normalOffset[1] * imageSize)),
-                        getScalarFromRGB(rgbArray.GetTuple(idx + normalOffset[2] * imageSize))
+                        getScalarFromRGB(
+                            rgbArray.GetTuple(idx + normalOffset[0] * imageSize)
+                        ),
+                        getScalarFromRGB(
+                            rgbArray.GetTuple(idx + normalOffset[1] * imageSize)
+                        ),
+                        getScalarFromRGB(
+                            rgbArray.GetTuple(idx + normalOffset[2] * imageSize)
+                        ),
                     )
 
                     # Re-orient normal to be view based
                     vect = normalByLayer.GetTuple3(layerIdx * imageSize + idx)
                     if not math.isnan(vect[0]):
                         # Express normal in new basis we computed above
-                        rVect = normalize([ -dotProduct(vect, camRight), dotProduct(vect, camUp), dotProduct(vect, camDir)  ])
+                        rVect = normalize(
+                            [
+                                -dotProduct(vect, camRight),
+                                dotProduct(vect, camUp),
+                                dotProduct(vect, camDir),
+                            ]
+                        )
 
                         # Need to reverse vector ?
                         if rVect[2] < 0:
-                            normalByLayer.SetTuple3(layerIdx * imageSize + idx, -rVect[0], -rVect[1], -rVect[2])
+                            normalByLayer.SetTuple3(
+                                layerIdx * imageSize + idx,
+                                -rVect[0],
+                                -rVect[1],
+                                -rVect[2],
+                            )
                         else:
-                            normalByLayer.SetTuple3(layerIdx * imageSize + idx, rVect[0], rVect[1], rVect[2])
+                            normalByLayer.SetTuple3(
+                                layerIdx * imageSize + idx, rVect[0], rVect[1], rVect[2]
+                            )
 
                 layerIdx += 1
 
@@ -243,25 +272,51 @@ class ConvertCompositeSpriteToSortedStack(object):
                     offset = layerIdx * imageSize
                     imageIdx = idx % imageSize
                     vect = normalByLayer.GetTuple3(imageIdx + offset)
-                    if not math.isnan(vect[0]) and not math.isnan(vect[1]) and not math.isnan(vect[2]):
-                        sortedNormal.SetTuple3(idx, int(127.5 * (vect[0] + 1)), int(127.5 * (vect[1] + 1)), int(255 * vect[2]))
+                    if (
+                        not math.isnan(vect[0])
+                        and not math.isnan(vect[1])
+                        and not math.isnan(vect[2])
+                    ):
+                        sortedNormal.SetTuple3(
+                            idx,
+                            int(127.5 * (vect[0] + 1)),
+                            int(127.5 * (vect[1] + 1)),
+                            int(255 * vect[2]),
+                        )
                     else:
-                        print ('WARNING: encountered NaN in normal of layer ',layerIdx,': [',vect[0],',',vect[1],',',vect[2],']')
+                        print(
+                            "WARNING: encountered NaN in normal of layer ",
+                            layerIdx,
+                            ": [",
+                            vect[0],
+                            ",",
+                            vect[1],
+                            ",",
+                            vect[2],
+                            "]",
+                        )
                         sortedNormal.SetTuple3(idx, 128, 128, 255)
 
             # Write the sorted data
-            with open(os.path.join(directory, 'normal.uint8'), 'wb') as f:
-                f.write(buffer(sortedNormal))
-                self.data.append({'name': 'normal', 'type': 'array', 'fileName': '/normal.uint8', 'categories': ['normal']})
+            with open(os.path.join(directory, "normal.uint8"), "wb") as f:
+                f.write(memoryview(sortedNormal))
+                self.data.append(
+                    {
+                        "name": "normal",
+                        "type": "array",
+                        "fileName": "/normal.uint8",
+                        "categories": ["normal"],
+                    }
+                )
 
         # Encode Intensity (sorted order way)
-        if 'intensity' in self.layers[0]:
+        if "intensity" in self.layers[0]:
             intensityOffsets = []
             sortedIntensity = vtkUnsignedCharArray()
             sortedIntensity.SetNumberOfTuples(stackSize)
 
             for layer in self.layers:
-                intensityOffsets.append(layer['intensity'])
+                intensityOffsets.append(layer["intensity"])
 
             for idx in range(stackSize):
                 layerIdx = int(orderArray.GetValue(idx))
@@ -270,20 +325,33 @@ class ConvertCompositeSpriteToSortedStack(object):
                 else:
                     offset = 3 * intensityOffsets[layerIdx] * imageSize
                     imageIdx = idx % imageSize
-                    sortedIntensity.SetValue(idx, rgbArray.GetValue(imageIdx * 3 + offset))
+                    sortedIntensity.SetValue(
+                        idx, rgbArray.GetValue(imageIdx * 3 + offset)
+                    )
 
-            with open(os.path.join(directory, 'intensity.uint8'), 'wb') as f:
-                f.write(buffer(sortedIntensity))
-                self.data.append({'name': 'intensity', 'type': 'array', 'fileName': '/intensity.uint8', 'categories': ['intensity']})
+            with open(os.path.join(directory, "intensity.uint8"), "wb") as f:
+                f.write(memoryview(sortedIntensity))
+                self.data.append(
+                    {
+                        "name": "intensity",
+                        "type": "array",
+                        "fileName": "/intensity.uint8",
+                        "categories": ["intensity"],
+                    }
+                )
 
         # Encode Each layer Scalar
         layerIdx = 0
         for layer in self.layers:
             for scalar in layer:
-                if scalar not in ['intensity', 'normal']:
+                if scalar not in ["intensity", "normal"]:
                     offset = imageSize * layer[scalar]
-                    scalarRange = self.config['scene'][layerIdx]['colors'][scalar]['range']
-                    delta = (scalarRange[1] - scalarRange[0]) / 16777215.0 # 2^24 - 1 => 16,777,215
+                    scalarRange = self.config["scene"][layerIdx]["colors"][scalar][
+                        "range"
+                    ]
+                    delta = (
+                        scalarRange[1] - scalarRange[0]
+                    ) / 16777215.0  # 2^24 - 1 => 16,777,215
 
                     scalarArray = vtkFloatArray()
                     scalarArray.SetNumberOfTuples(imageSize)
@@ -291,16 +359,27 @@ class ConvertCompositeSpriteToSortedStack(object):
                         rgb = rgbArray.GetTuple(idx + offset)
                         if rgb[0] != 0 or rgb[1] != 0 or rgb[2] != 0:
                             # Decode encoded value
-                            value = scalarRange[0] + delta * float(rgb[0]*65536 + rgb[1]*256 + rgb[2] - 1)
+                            value = scalarRange[0] + delta * float(
+                                rgb[0] * 65536 + rgb[1] * 256 + rgb[2] - 1
+                            )
                             scalarArray.SetValue(idx, value)
                         else:
                             # No value
-                            scalarArray.SetValue(idx, float('NaN'))
+                            scalarArray.SetValue(idx, float("NaN"))
 
-
-                    with open(os.path.join(directory, '%d_%s.float32' % (layerIdx, scalar)), 'wb') as f:
-                        f.write(buffer(scalarArray))
-                        self.data.append({'name': '%d_%s' % (layerIdx, scalar), 'type': 'array', 'fileName': '/%d_%s.float32' % (layerIdx, scalar), 'categories': ['%d_%s' % (layerIdx, scalar)]})
+                    with open(
+                        os.path.join(directory, "%d_%s.float32" % (layerIdx, scalar)),
+                        "wb",
+                    ) as f:
+                        f.write(memoryview(scalarArray))
+                        self.data.append(
+                            {
+                                "name": "%d_%s" % (layerIdx, scalar),
+                                "type": "array",
+                                "fileName": "/%d_%s.float32" % (layerIdx, scalar),
+                                "categories": ["%d_%s" % (layerIdx, scalar)],
+                            }
+                        )
 
             layerIdx += 1
 
@@ -309,6 +388,7 @@ class ConvertCompositeSpriteToSortedStack(object):
 # Composite Sprite to Sorted Composite Dataset Builder
 # -----------------------------------------------------------------------------
 
+
 class ConvertCompositeDataToSortedStack(object):
     def __init__(self, directory):
         self.basePath = directory
@@ -319,7 +399,7 @@ class ConvertCompositeDataToSortedStack(object):
         # Load JSON metadata
         with open(os.path.join(directory, "config.json"), "r") as f:
             self.config = json.load(f)
-            self.nbLayers = len(self.config['scene'])
+            self.nbLayers = len(self.config["scene"])
             while len(self.layers) < self.nbLayers:
                 self.layers.append({})
 
@@ -331,26 +411,28 @@ class ConvertCompositeDataToSortedStack(object):
 
     def convert(self):
         for root, dirs, files in os.walk(self.basePath):
-            if 'depth_0.float32' in files:
-                print ('Process', root)
+            if "depth_0.float32" in files:
+                print("Process", root)
                 self.processDirectory(root)
 
     def processDirectory(self, directory):
         # Load depth
         depthStack = []
-        imageSize = self.config['size']
+        imageSize = self.config["size"]
         linearSize = imageSize[0] * imageSize[1]
         nbLayers = len(self.layers)
         stackSize = nbLayers * linearSize
         layerList = range(nbLayers)
         for layerIdx in layerList:
-            with open(os.path.join(directory, 'depth_%d.float32' % layerIdx), "rb") as f:
-                a = array.array('f')
+            with open(
+                os.path.join(directory, "depth_%d.float32" % layerIdx), "rb"
+            ) as f:
+                a = array.array("f")
                 a.fromfile(f, linearSize)
                 depthStack.append(a)
 
         orderArray = vtkUnsignedCharArray()
-        orderArray.SetName('layerIdx');
+        orderArray.SetName("layerIdx")
         orderArray.SetNumberOfComponents(1)
         orderArray.SetNumberOfTuples(stackSize)
 
@@ -369,31 +451,36 @@ class ConvertCompositeDataToSortedStack(object):
 
             # Fill sortedOrder array
             for layerIdx in layerList:
-                orderArray.SetValue(layerIdx * linearSize + pixelId, pixelSorter[layerIdx][0])
+                orderArray.SetValue(
+                    layerIdx * linearSize + pixelId, pixelSorter[layerIdx][0]
+                )
 
         # Write order (sorted order way)
-        with open(os.path.join(directory, 'order.uint8'), 'wb') as f:
-            f.write(buffer(orderArray))
-            self.data.append({'name': 'order', 'type': 'array', 'fileName': '/order.uint8'})
+        with open(os.path.join(directory, "order.uint8"), "wb") as f:
+            f.write(memoryview(orderArray))
+            self.data.append(
+                {"name": "order", "type": "array", "fileName": "/order.uint8"}
+            )
 
         # Remove depth files
         for layerIdx in layerList:
-            os.remove(os.path.join(directory, 'depth_%d.float32' % layerIdx))
-
+            os.remove(os.path.join(directory, "depth_%d.float32" % layerIdx))
 
         # Encode Normals (sorted order way)
-        if 'normal' in self.config['light']:
+        if "normal" in self.config["light"]:
             sortedNormal = vtkUnsignedCharArray()
-            sortedNormal.SetNumberOfComponents(3) # x,y,z
+            sortedNormal.SetNumberOfComponents(3)  # x,y,z
             sortedNormal.SetNumberOfTuples(stackSize)
 
             # Get Camera orientation and rotation information
-            camDir = [0,0,0]
-            worldUp = [0,0,0]
+            camDir = [0, 0, 0]
+            worldUp = [0, 0, 0]
             with open(os.path.join(directory, "camera.json"), "r") as f:
                 camera = json.load(f)
-                camDir = normalize([ camera['position'][i] - camera['focalPoint'][i] for i in range(3) ])
-                worldUp = normalize(camera['viewUp'])
+                camDir = normalize(
+                    [camera["position"][i] - camera["focalPoint"][i] for i in range(3)]
+                )
+                worldUp = normalize(camera["viewUp"])
 
             # [ camRight, camUp, camDir ] will be our new orthonormal basis for normals
             camRight = vectProduct(camDir, worldUp)
@@ -412,8 +499,13 @@ class ConvertCompositeDataToSortedStack(object):
                 # Load normal(x,y,z) from current layer
                 normalLayer = []
                 for comp in [0, 1, 2]:
-                    with open(os.path.join(directory, 'normal_%d_%d.float32' % (layerIdx, comp)), "rb") as f:
-                        a = array.array('f')
+                    with open(
+                        os.path.join(
+                            directory, "normal_%d_%d.float32" % (layerIdx, comp)
+                        ),
+                        "rb",
+                    ) as f:
+                        a = array.array("f")
                         a.fromfile(f, linearSize)
                         normalLayer.append(a)
 
@@ -424,20 +516,36 @@ class ConvertCompositeDataToSortedStack(object):
                         idx + offset,
                         normalLayer[0][idx],
                         normalLayer[1][idx],
-                        normalLayer[2][idx]
+                        normalLayer[2][idx],
                     )
 
                     # Re-orient normal to be view based
                     vect = normalByLayer.GetTuple3(layerIdx * linearSize + idx)
                     if not math.isnan(vect[0]):
                         # Express normal in new basis we computed above
-                        rVect = normalize([ -dotProduct(vect, camRight), dotProduct(vect, camUp), dotProduct(vect, camDir)  ])
+                        rVect = normalize(
+                            [
+                                -dotProduct(vect, camRight),
+                                dotProduct(vect, camUp),
+                                dotProduct(vect, camDir),
+                            ]
+                        )
 
                         # Need to reverse vector ?
                         if rVect[2] < 0:
-                            normalByLayer.SetTuple3(layerIdx * linearSize + idx, -rVect[0], -rVect[1], -rVect[2])
+                            normalByLayer.SetTuple3(
+                                layerIdx * linearSize + idx,
+                                -rVect[0],
+                                -rVect[1],
+                                -rVect[2],
+                            )
                         else:
-                            normalByLayer.SetTuple3(layerIdx * linearSize + idx, rVect[0], rVect[1], rVect[2])
+                            normalByLayer.SetTuple3(
+                                layerIdx * linearSize + idx,
+                                rVect[0],
+                                rVect[1],
+                                rVect[2],
+                            )
 
             # Sort normals and encode them as 3 bytes ( -1 < xy < 1 | 0 < z < 1)
             for idx in range(stackSize):
@@ -449,32 +557,66 @@ class ConvertCompositeDataToSortedStack(object):
                     offset = layerIdx * linearSize
                     imageIdx = idx % linearSize
                     vect = normalByLayer.GetTuple3(imageIdx + offset)
-                    if not math.isnan(vect[0]) and not math.isnan(vect[1]) and not math.isnan(vect[2]):
-                        sortedNormal.SetTuple3(idx, int(127.5 * (vect[0] + 1)), int(127.5 * (vect[1] + 1)), int(255 * vect[2]))
+                    if (
+                        not math.isnan(vect[0])
+                        and not math.isnan(vect[1])
+                        and not math.isnan(vect[2])
+                    ):
+                        sortedNormal.SetTuple3(
+                            idx,
+                            int(127.5 * (vect[0] + 1)),
+                            int(127.5 * (vect[1] + 1)),
+                            int(255 * vect[2]),
+                        )
                     else:
-                        print ('WARNING: encountered NaN in normal of layer ',layerIdx,': [',vect[0],',',vect[1],',',vect[2],']')
+                        print(
+                            "WARNING: encountered NaN in normal of layer ",
+                            layerIdx,
+                            ": [",
+                            vect[0],
+                            ",",
+                            vect[1],
+                            ",",
+                            vect[2],
+                            "]",
+                        )
                         sortedNormal.SetTuple3(idx, 128, 128, 255)
 
             # Write the sorted data
-            with open(os.path.join(directory, 'normal.uint8'), 'wb') as f:
-                f.write(buffer(sortedNormal))
-                self.data.append({'name': 'normal', 'type': 'array', 'fileName': '/normal.uint8', 'categories': ['normal']})
+            with open(os.path.join(directory, "normal.uint8"), "wb") as f:
+                f.write(memoryview(sortedNormal))
+                self.data.append(
+                    {
+                        "name": "normal",
+                        "type": "array",
+                        "fileName": "/normal.uint8",
+                        "categories": ["normal"],
+                    }
+                )
 
             # Remove depth files
             for layerIdx in layerList:
-                os.remove(os.path.join(directory, 'normal_%d_%d.float32' % (layerIdx, 0)))
-                os.remove(os.path.join(directory, 'normal_%d_%d.float32' % (layerIdx, 1)))
-                os.remove(os.path.join(directory, 'normal_%d_%d.float32' % (layerIdx, 2)))
+                os.remove(
+                    os.path.join(directory, "normal_%d_%d.float32" % (layerIdx, 0))
+                )
+                os.remove(
+                    os.path.join(directory, "normal_%d_%d.float32" % (layerIdx, 1))
+                )
+                os.remove(
+                    os.path.join(directory, "normal_%d_%d.float32" % (layerIdx, 2))
+                )
 
         # Encode Intensity (sorted order way)
-        if 'intensity' in self.config['light']:
+        if "intensity" in self.config["light"]:
             sortedIntensity = vtkUnsignedCharArray()
             sortedIntensity.SetNumberOfTuples(stackSize)
 
             intensityLayers = []
             for layerIdx in layerList:
-                with open(os.path.join(directory, 'intensity_%d.uint8' % layerIdx), "rb") as f:
-                    a = array.array('B')
+                with open(
+                    os.path.join(directory, "intensity_%d.uint8" % layerIdx), "rb"
+                ) as f:
+                    a = array.array("B")
                     a.fromfile(f, linearSize)
                     intensityLayers.append(a)
 
@@ -486,10 +628,17 @@ class ConvertCompositeDataToSortedStack(object):
                     imageIdx = idx % linearSize
                     sortedIntensity.SetValue(idx, intensityLayers[layerIdx][imageIdx])
 
-            with open(os.path.join(directory, 'intensity.uint8'), 'wb') as f:
-                f.write(buffer(sortedIntensity))
-                self.data.append({'name': 'intensity', 'type': 'array', 'fileName': '/intensity.uint8', 'categories': ['intensity']})
+            with open(os.path.join(directory, "intensity.uint8"), "wb") as f:
+                f.write(memoryview(sortedIntensity))
+                self.data.append(
+                    {
+                        "name": "intensity",
+                        "type": "array",
+                        "fileName": "/intensity.uint8",
+                        "categories": ["intensity"],
+                    }
+                )
 
             # Remove depth files
             for layerIdx in layerList:
-                os.remove(os.path.join(directory, 'intensity_%d.uint8' % layerIdx))
+                os.remove(os.path.join(directory, "intensity_%d.uint8" % layerIdx))
